package de.jottyfan.timetrack.modules.done;

import java.time.Duration;
import java.time.LocalDate;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import de.jottyfan.timetrack.component.OAuth2Provider;
import de.jottyfan.timetrack.modules.CommonController;
import de.jottyfan.timetrack.modules.done.model.DoneBean;
import de.jottyfan.timetrack.modules.done.model.DoneModel;
import de.jottyfan.timetrack.modules.done.model.OvertimeBean;
import de.jottyfan.timetrack.modules.done.model.SlotBean;
import de.jottyfan.timetrack.modules.done.model.SlotRangeBean;
import de.jottyfan.timetrack.modules.done.model.SummaryBean;
import jakarta.annotation.security.RolesAllowed;
import jakarta.websocket.server.PathParam;

/**
 * 
 * @author henkej
 *
 */
@Controller
@SessionAttributes("doneModel")
public class DoneController extends CommonController {

	@Autowired
	private OAuth2Provider provider;

	@Autowired
	private DoneService doneService;

	@ModelAttribute("doneModel")
	DoneModel getdoneModel() {
		return new DoneModel();
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/list")
	public String getList(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		String username = provider.getName();
		Duration maxWorkTime = Duration.ofHours(8); // TODO: to the configuration file
		LocalDate day = doneModel.getDay();
		List<DoneBean> list = doneService.getList(day, username);
		List<DoneBean> week = doneService.getWeek(day, username);
		SummaryBean sumBean = new SummaryBean(list, day, maxWorkTime);
		SummaryBean weekBean = new SummaryBean(week, day, maxWorkTime);
		model.addAttribute("doneList", list);
		Duration sumtimeDuration = Duration.ofMinutes(0);
		for (DoneBean bean : list) {
			sumtimeDuration = sumtimeDuration.plus(bean.getTimeDiffDuration());
		}
		model.addAttribute("sumtime", String.format("%02d:%02d", sumtimeDuration.toHours(), sumtimeDuration.toMinutes() % 60));
		model.addAttribute("sum", sumBean);
		model.addAttribute("daysum", doneService.getDaysum(day, username));
		model.addAttribute("overtimeBean", doneService.getOvertimeBean(username));
		model.addAttribute("slots", doneService.getSlots(day, username));
		model.addAttribute("slotOffset", doneService.getSlotOffset(day));
		model.addAttribute("schedule", weekBean.toJson());
		model.addAttribute("recentList", doneService.getListRecent(username, 10));
		model.addAttribute("projectList", doneService.getProjects(false));
		model.addAttribute("moduleList", doneService.getModules(false));
		model.addAttribute("jobList", doneService.getJobs(false));
		model.addAttribute("billingList", doneService.getBillings(false));
		model.addAttribute("favorites", doneService.getFavorites(username));
		return "done/list";
	}

	@RolesAllowed("timetrack_user")
	@PostMapping("/done/list")
	public String getListForDate(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		return getList(model, doneModel);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/update/{id}")
	public String updateField(@PathVariable("id") Integer fkDone, @PathParam("field") String field, @PathParam("value") Integer value) {
		doneService.updateField(fkDone, field, value);
		return "redirect:/done/list";		
	}
	
	@RolesAllowed("timetrack_user")
	@GetMapping("/done/abort/{day}")
	public String abort(@PathVariable("day") String day, Model model) {
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/add/{day}")
	public String toAdd(@PathVariable("day") @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate day, Model model) {
		DoneBean bean = new DoneBean();
		bean.setLocalDate(day);
		return toItem(bean, model);
	}

	private String toItem(DoneBean bean, Model model) {
		model.addAttribute("doneBean", bean);
		model.addAttribute("projectList", doneService.getProjects(true));
		model.addAttribute("moduleList", doneService.getModules(true));
		model.addAttribute("jobList", doneService.getJobs(true));
		model.addAttribute("billingList", doneService.getBillings(true));
		return "done/item";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/edit/{id}")
	public String toItem(@PathVariable("id") Integer id, Model model) {
		DoneBean bean = doneService.getBean(id);
		if (bean == null) {
			bean = new DoneBean(); // the add case; typically, only add from today
		}
		return toItem(bean, model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/end/{id}")
	public String end(@PathVariable("id") Integer id, Model model) {
		DoneBean bean = doneService.getBean(id);
		String username = provider.getName();
		doneService.endToNow(bean, username);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/copy/{id}")
	public String copyFromNow(@PathVariable("id") Integer id, Model model) {
		DoneBean bean = doneService.getBean(id);
		String username = provider.getName();
		doneService.copyFromNow(bean, username);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@PostMapping("/done/upsert")
	public String doUpsert(Model model, @ModelAttribute("bean") DoneBean bean) {
		String username = provider.getName();
		Integer amount = doneService.doUpsert(bean, username);
		return amount.equals(1) ? "redirect:/done/list" : "redirect:/" + toItem(bean.getPk(), model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/addrecent/{id}")
	public String addRecent(Model model, @PathVariable("id") Integer id) {
		String username = provider.getName();
		DoneBean bean = doneService.getBean(id);
		doneService.addRecent(bean, username);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/list/previousday")
	public String previousDay(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		LocalDate day = doneModel.getDay();
		doneModel.setDay(day.minusDays(1));
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/list/nextday")
	public String nextDay(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		LocalDate day = doneModel.getDay();
		doneModel.setDay(day.plusDays(1));
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/delete/{id}")
	public String doDelete(@PathVariable("id") Integer id, Model model) {
		Integer amount = doneService.doDelete(id);
		return amount.equals(1) ? "redirect:/done/list" : "redirect:/" + toItem(id, model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/favorize/{id}")
	public String favorize(@PathVariable("id") Integer id) {
		doneService.favorize(id);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/unfavorize/{id}")
	public String unfavorize(@PathVariable("id") Integer id) {
		doneService.unfavorize(id);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/usefav/{id}")
	public String usefavorite(@PathVariable("id") Integer id) {
		doneService.usefavorite(id);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@PostMapping(value = "/done/overtime/update")
	public String upsertOvertime(@ModelAttribute("overtimeBean") OvertimeBean bean) {
		String username = provider.getName();
		doneService.upsertOvertime(bean, username);
		return "redirect:/done/list";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/{id}")
	public String loadSlot(@PathVariable("id") Integer id, Model model) {
		String username = provider.getName();
		model.addAttribute("bean", doneService.getSlot(id, username));
		return "/done/slot/item";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/add")
	public String addSlot(@RequestParam("day") LocalDate day, Model model) {
		model.addAttribute("bean", SlotBean.of(day));
		return "/done/slot/item";
	}

	@RolesAllowed("timetrack_user")
	@PostMapping("/done/slot/upsert")
	public String upsertSlot(@ModelAttribute("bean") SlotBean bean, Model model) {
		doneService.upsert(bean, provider.getName());
		return "redirect:/done/list#div_slot";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/{id}/delete")
	public String deleteSlot(@PathVariable("id") Integer slotId) {
		doneService.delete(slotId, provider.getName());
		return "redirect:/done/list#div_slot";
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/range")
	public String toAddRange(Model model) {
		model.addAttribute("bean", new SlotRangeBean());
		return "/done/slot/range";
	}
	
	@RolesAllowed("timetrack_user")
	@PostMapping("/done/slot/addrange")
	public String addRange(@ModelAttribute("bean") SlotRangeBean bean) {
		doneService.addSlotRange(bean.getMinutes(), bean.getFrom(), bean.getUntil(), bean.getReason(), provider.getName(),
				bean.getIncludeSaturday(), bean.getIncludeSunday());
		return "redirect:/done/list#div_slot";
	}
	
	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/back")
	public String oneMonthBack(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		LocalDate day = doneModel.getDay();
		doneModel.setDay(day.minusMonths(1));
		return "redirect:/done/list#div_slot";
	}
	
	@RolesAllowed("timetrack_user")
	@GetMapping("/done/slot/forward")
	public String oneMonthForward(Model model, @ModelAttribute("doneModel") DoneModel doneModel) {
		LocalDate day = doneModel.getDay();
		doneModel.setDay(day.plusMonths(1));
		return "redirect:/done/list#div_slot";
	}
}
