package de.jottyfan.timetrack.modules.done;

import static de.jottyfan.timetrack.db.done.Tables.T_DONE;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClient;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClientService;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import de.jottyfan.timetrack.db.done.tables.records.VBillingRecord;
import de.jottyfan.timetrack.db.done.tables.records.VJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.VModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.VProjectRecord;
import de.jottyfan.timetrack.modules.done.model.DaysumBean;
import de.jottyfan.timetrack.modules.done.model.DoneBean;
import de.jottyfan.timetrack.modules.done.model.FavoriteBean;
import de.jottyfan.timetrack.modules.done.model.OvertimeBean;
import de.jottyfan.timetrack.modules.done.model.SlotBean;
import de.jottyfan.timetrack.modules.note.NoteService;

/**
 * 
 * @author henkej
 *
 */
@Service
@Transactional(transactionManager = "transactionManager")
public class DoneService {
	private static final Logger LOGGER = LogManager.getLogger(NoteService.class);
	private static final int INTERVAL = 15;

	@Autowired
	private TimeService timeService;

	@Autowired
	private DoneRepository repository;

	@Autowired
	private DSLContext dsl;

	@Autowired
	private OAuth2AuthorizedClientService security;

	public String getCurrentUser(OAuth2AuthenticationToken token) {
		if (token != null) {
			OAuth2AuthorizedClient client = security.loadAuthorizedClient(token.getAuthorizedClientRegistrationId(),
					token.getName());
			if (client != null) {
				return client.getPrincipalName();
			} else {
				return "client is null";
			}
		} else {
			return "oauth token is null";
		}
	}

	public List<DoneBean> getList(LocalDate day, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			if (userId == null) {
				LOGGER.warn("userId of user {} is null", username);
			}
			return gw.getAllOfDay(day, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public List<DoneBean> getWeek(LocalDate day, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			if (userId == null) {
				LOGGER.warn("userId of user {} is null", username);
			}
			return gw.getAllOfWeek(day, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public DoneBean getBean(Integer id) {
		try {
			return new DoneGateway(dsl).getBean(id);
		} catch (Exception e) {
			LOGGER.error(e);
			return null;
		}
	}

	public List<VProjectRecord> getProjects(boolean includeNull) {
		try {
			return new DoneGateway(dsl).getAllProjects(includeNull);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public List<VModuleRecord> getModules(boolean includeNull) {
		try {
			return new DoneGateway(dsl).getAllModules(includeNull);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public List<VJobRecord> getJobs(boolean includeNull) {
		try {
			return new DoneGateway(dsl).getAllJobs(includeNull);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public List<VBillingRecord> getBillings(boolean includeNull) {
		try {
			return new DoneGateway(dsl).getAllBillings(includeNull);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public Integer doUpsert(DoneBean bean, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			return gw.upsert(bean, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

	public Integer doDelete(Integer id) {
		try {
			return new DoneGateway(dsl).delete(id);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

	public Integer endToNow(DoneBean bean, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			bean.setTimeUntil(timeService.roundTime(LocalDateTime.now(), INTERVAL));
			return gw.upsert(bean, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

	public Integer copyFromNow(DoneBean bean, String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			bean.setTimeFrom(timeService.roundTime(LocalDateTime.now(), INTERVAL));
			bean.setTimeUntil(null);
			bean.setPk(null);
			return gw.upsert(bean, userId);
		} catch (Exception e) {
			LOGGER.error(e);
			return -1;
		}
	}

	public List<DoneBean> getListRecent(String username, int recentCount) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer userId = gw.getUserId(username);
			if (userId == null) {
				LOGGER.warn("userId of user {} is null", username);
			}
			return gw.getRecent(userId, recentCount);
		} catch (Exception e) {
			LOGGER.error(e);
			return new ArrayList<>();
		}
	}

	public Integer addRecent(DoneBean bean, String username) {
		bean.setPk(null);
		bean.setTimeFrom(timeService.roundTime(LocalDateTime.now(), INTERVAL));
		bean.setTimeUntil(null);
		return this.doUpsert(bean, username);
	}

	public void favorize(Integer id) {
		try {
			new DoneGateway(dsl).favorize(id);
		} catch (Exception e) {
		}
	}

	public void unfavorize(Integer id) {
		try {
			new DoneGateway(dsl).unfavorize(id);
		} catch (Exception e) {
		}
	}

	public List<FavoriteBean> getFavorites(String username) {
		try {
			DoneGateway gw = new DoneGateway(dsl);
			Integer login = gw.getUserId(username);
			return gw.getFavorites(login);
		} catch (Exception e) {
			return new ArrayList<>();
		}
	}

	public void usefavorite(Integer fkFavorite) {
		try {
			new DoneGateway(dsl).useFav(fkFavorite, timeService.roundTime(LocalDateTime.now(), INTERVAL));
		} catch (Exception e) {
		}
	}

	public DaysumBean getDaysum(LocalDate day, String login) {
		return repository.getDaysum(day, login);
	}

	public OvertimeBean getOvertimeBean(String login) {
		return repository.getOvertimeBean(login);
	}

	public void upsertOvertime(OvertimeBean bean, String username) {
		repository.upsertOvertime(bean.getId(), username, bean.getImpact(), bean.getOvertimeMinutes());
	}

	public List<SlotBean> getSlots(LocalDate day, String username) {
		YearMonth ym = YearMonth.from(day);
		LocalDate from = ym.atDay(1);
		LocalDate until = ym.atEndOfMonth();
		Map<LocalDate, SlotBean> map = new HashMap<>();
		LocalDate i = from;
		while (i.isBefore(until.plusDays(1))) {
			map.put(i, SlotBean.of(i));
			i = i.plusDays(1);
		}
		map.putAll(repository.getSlots(from, until, username));
		List<SlotBean> list = new ArrayList<>(map.values());
		list.sort((o1, o2) -> {
			return o1 == null || o2 == null || o1.getDay() == null ? 0 : o1.getDay().compareTo(o2.getDay());
		});
		return list;
	}

	public SlotBean getSlot(Integer id, String username) {
		return repository.getSlot(id, username);
	}

	/**
	 * get a list of days until the 1st of the month starts in the calendar - start
	 * with sunday for the matrix
	 * 
	 * @param day the day; only the month will be used
	 * @return a list of numbers
	 */
	public List<Integer> getSlotOffset(LocalDate day) {
		List<Integer> list = new ArrayList<Integer>();
		YearMonth ym = YearMonth.from(day);
		LocalDate first = ym.atDay(1);
		for (int i = 0; i < first.getDayOfWeek().getValue(); i++) {
			list.add(i);
		}
		return list;
	}

	/**
	 * upsert the bean
	 * 
	 * @param bean the bean
	 * @param username the username
	 */
	public void upsert(SlotBean bean, String username) {
		if (bean.getId() == null) {
			repository.addSlot(bean, username);
		} else {
			repository.updateSlot(bean, username);
		}
	}
	
	public void delete(Integer slotId, String username) {
		repository.deleteSlot(slotId, username);
	}
	
	public void addSlotRange(Integer minutes, LocalDate from, LocalDate until, String reason, String username, Boolean includeSaturdays, Boolean includeSundays) {
		List<LocalDate> days = new ArrayList<>();
		if (!from.isBefore(until)) {
			LocalDate tmp = from;
			from = until;
			until = tmp;
		}
		includeSaturdays = includeSaturdays == null ? false : includeSaturdays;
		includeSundays = includeSundays == null ? false : includeSundays;
		for (LocalDate i = from; i.isBefore(until.plusDays(1)); i = i.plusDays(1)) {
			if (i.getDayOfWeek().equals(DayOfWeek.SUNDAY) && !includeSundays) {
				// ignore
			} else if (i.getDayOfWeek().equals(DayOfWeek.SATURDAY) && !includeSaturdays) {
				// ignore
		  } else {
				days.add(i);
			}
		}
		repository.addSlotRange(minutes, username, reason, days);
	}

	public void updateField(Integer fkDone, String field, Integer value) {
		if ("project".equals(field)) {
			repository.updateField(fkDone, value, T_DONE.FK_PROJECT);
		} else if ("module".equals(field)) {
			repository.updateField(fkDone, value, T_DONE.FK_MODULE);
		} else if ("job".equals(field)) {
			repository.updateField(fkDone, value, T_DONE.FK_JOB);
		} else {
			LOGGER.error("field {} not supported yet", field);
		}
	}
}
