package de.jottyfan.timetrack.modules.done.project;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;

import de.jottyfan.timetrack.component.OAuth2Provider;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.modules.CommonController;
import de.jottyfan.timetrack.modules.profile.ProfileService;
import jakarta.annotation.security.RolesAllowed;

/**
 * 
 * @author henkej
 *
 */
@Controller
public class ProjectController extends CommonController {
	@Autowired
	private ProjectService projectService;

	@Autowired
	private OAuth2Provider provider;
	
	@Autowired
	private ProfileService profileService;

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/edit/project/{id}")
	public String toProject(@PathVariable("id") Integer id, Model model) {
		String username = provider.getName();
		TProjectRecord project = projectService.get(id);
		model.addAttribute("projectBean", project);
		model.addAttribute("theme", profileService.getTheme(username));
		return "done/project";
	}
	
	@RolesAllowed("timetrack_user")
	@PostMapping("/done/upsert/project")
	public String doUpsert(Model model, @ModelAttribute("bean") TProjectRecord bean) {
		Integer amount = projectService.doUpsert(bean);
		return amount.equals(1) ? "redirect:/done/list" : toProject(bean.getPk(), model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping("/done/add/project")
	public String toAddProject(Model model) {
		return toProject(null, model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/done/delete/project/{id}")
	public String doDeleteProject(@PathVariable("id") Integer id, Model model) {
		Integer amount = projectService.doDelete(id);
		return amount.equals(1) ? "redirect:/done/list" : toProject(id, model);
	}
}
